from ekp_sdk.util.clean_null_terms import clean_null_terms


def Alert(content, header=None, icon_name=None, class_name=None, style=None, when=None):
    return {
        "_type": "Alert",
        "props": clean_null_terms({
            "content": content,
            "header": header,
            "iconName": icon_name,
            "className": class_name,
            "style": style,
            "when": when,
        })
    }


from ekp_sdk.util.clean_null_terms import clean_null_terms


def Timeline(events=None, title=None, content=None, style=None, class_name = None):
    return {
        "_type": "Timeline",
        "props": clean_null_terms({
            "className": class_name,
            "events": events or [],
            "title": title,
            "content": content,
            "style": style,
        })
    }


from ekp_sdk.util.clean_null_terms import clean_null_terms


def Span(content, class_name=None, when=None):
    return {
        "_type": "Span",
        "props": clean_null_terms({
            "className": class_name,
            "content": content,
            "when": when
        })
    }


from ekp_sdk.util.clean_null_terms import clean_null_terms


def Datatable(
    data,
    columns,
    busy_when=None,
    class_name=None,
    default_sort_asc=None,
    default_sort_field_id=None,
    default_view=None,
    disable_list_view=None,
    filters=None,
    grid_view=None,
    on_row_clicked=None,
    pagination=None,
    pagination_per_page=None,
    search_hint=None,
    show_export=None,
    show_last_updated=None,
    card=None
):
    return {
        "_type": "Datatable",
        "props": clean_null_terms({
            "busyWhen": busy_when,
            "card": card,
            "className": class_name,
            "columns": columns,
            "data": data,
            "defaultSortAsc": default_sort_asc,
            "defaultSortFieldId": default_sort_field_id,
            "defaultView": default_view,
            "disableListView": disable_list_view,
            "filters": filters,
            "gridView": grid_view,
            "onRowClicked": on_row_clicked,
            "pagination": pagination,
            "paginationPerPage": pagination_per_page,
            "searchHint": search_hint,
            "showExport": show_export,
            "showLastUpdated": show_last_updated,
        })
    }


from ekp_sdk.util.clean_null_terms import clean_null_terms


def Select(label, name, options, min_width=None, class_name=None):
    return {
        "_type": "Select",
        "props": clean_null_terms({
            "className": class_name,
            "label": label,
            "name": name,
            "options": options,
            "minWidth": min_width,
        })
    }


from ekp_sdk.ui.components.chart import *
from ekp_sdk.ui.components.link import *
from ekp_sdk.ui.components.tabs import *
from ekp_sdk.ui.components.form import *
from ekp_sdk.ui.components.input import *
from ekp_sdk.ui.components.alert import *
from ekp_sdk.ui.components.avatar import *
from ekp_sdk.ui.components.badge import *
from ekp_sdk.ui.components.button import *
from ekp_sdk.ui.components.card import *
from ekp_sdk.ui.components.col import *
from ekp_sdk.ui.components.column import *
from ekp_sdk.ui.components.container import *
from ekp_sdk.ui.components.datatable import *
from ekp_sdk.ui.components.div import *
from ekp_sdk.ui.components.fragment import *
from ekp_sdk.ui.components.hr import *
from ekp_sdk.ui.components.icon import *
from ekp_sdk.ui.components.image import *
from ekp_sdk.ui.components.paragraphs import *
from ekp_sdk.ui.components.row import *
from ekp_sdk.ui.components.select import *
from ekp_sdk.ui.components.span import *
from ekp_sdk.ui.components.timeline import *
from ekp_sdk.ui.components.ul import *



from ekp_sdk.util.clean_null_terms import clean_null_terms


def Button(
    busy_when=None,
    class_name=None,
    color=None,
    icon=None,
    is_submit=None,
    label=None,
    on_click=None,
    size=None,
    tooltip=None
):
    return {
        "_type": "Button",
        "props": clean_null_terms({
            "busyWhen": busy_when,
            "className": class_name,
            "color": color,
            "icon": icon,
            "isSubmit": is_submit,
            "label": label,
            "onClick": on_click,
            "size": size,
            "tooltip": tooltip,
        })
    }


from ekp_sdk.db.contract_logs_repo import ContractLogsRepo
from ekp_sdk.db.contract_transactions_repo import ContractTransactionsRepo
from ekp_sdk.db.mg_client import MgClient
from ekp_sdk.db.pg_client import PgClient
from ekp_sdk.services.cache_service import CacheService
from ekp_sdk.services.client_service import ClientService
from ekp_sdk.services.coingecko_service import CoingeckoService
from ekp_sdk.services.etherscan_service import EtherscanService
from ekp_sdk.services.google_sheets_client import GoogleSheetsClient
from ekp_sdk.services.moralis_api_service import MoralisApiService
from ekp_sdk.services.redis_client import RedisClient
from ekp_sdk.services.rest_client import RestClient
from ekp_sdk.services.transaction_sync_service import TransactionSyncService
from ekp_sdk.services.twitter_client import TwitterClient
from ekp_sdk.services.web3_service import Web3Service
import logging


class BaseContainer:
    def __init__(self, config):

        # -----------------------------------------------------------------

        self.rest_client = RestClient()

        # -----------------------------------------------------------------

        self.coingecko_service = CoingeckoService(
            rest_client=self.rest_client
        )

        # -----------------------------------------------------------------

        REDIS_URI = config("REDIS_URI", default=None)

        if REDIS_URI is not None:
            self.redis_client = RedisClient(
                uri=REDIS_URI
            )
        else:
            logging.warn("⚠️ skipped RedisClient init, missing REDIS_URI")

        # -----------------------------------------------------------------

        POSTGRES_URI = config("POSTGRES_URI", default=None)

        if POSTGRES_URI is not None:
            self.pg_client = PgClient(
                uri=POSTGRES_URI,
            )
        else:
            logging.warn("⚠️ skipped PgClient init, missing POSTGRES_URI")

        # -----------------------------------------------------------------

        MONGO_URI = config('MONGO_URI', default=None)

        if MONGO_URI is not None:

            MONGO_DB_NAME = config('MONGO_DB_NAME')

            self.mg_client = MgClient(
                uri=MONGO_URI,
                db_name=MONGO_DB_NAME
            )
        else:
            logging.warn("⚠️ skipped MgClient init, missing MONGO_URI")

        # -----------------------------------------------------------------

        ETHERSCAN_API_KEY = config("ETHERSCAN_API_KEY", default=None)
        ETHERSCAN_BASE_URL = config("ETHERSCAN_BASE_URL", default=None)

        if ETHERSCAN_API_KEY is not None and ETHERSCAN_BASE_URL is not None:
            self.etherscan_service = EtherscanService(
                api_key=ETHERSCAN_API_KEY,
                base_url=ETHERSCAN_BASE_URL,
                rest_client=self.rest_client
            )

            if MONGO_URI is not None:
                self.contract_transactions_repo = ContractTransactionsRepo(
                    mg_client=self.mg_client,
                )

                self.contract_logs_repo = ContractLogsRepo(
                    mg_client=self.mg_client,
                )

                self.transaction_sync_service = TransactionSyncService(
                    contract_transactions_repo=self.contract_transactions_repo,
                    contract_logs_repo=self.contract_logs_repo,
                    etherscan_service=self.etherscan_service,
                )
            else:
                logging.warn(
                    "⚠️ skipped TransactionSyncService init, missing MONGO_URI"
                )
        else:
            logging.warn(
                "⚠️ skipped EtherscanService init, missing ETHERSCAN_API_KEY and ETHERSCAN_BASE_URL"
            )

        # -----------------------------------------------------------------

        MORALIS_API_KEY = config('MORALIS_API_KEY', default=None)

        if MORALIS_API_KEY is not None:
            self.moralis_api_service = MoralisApiService(
                api_key=MORALIS_API_KEY,
                rest_client=self.rest_client
            )
        else:
            logging.warn(
                "⚠️ skipped MoralisApiService init, missing MORALIS_API_KEY"
            )

        # -----------------------------------------------------------------

        if REDIS_URI is not None:
            self.cache_service = CacheService(
                redis_client=self.redis_client,
            )
        else:
            logging.warn("⚠️ skipped CacheService init, missing REDIS_URI")

        # -----------------------------------------------------------------

        WEB3_PROVIDER_URL = config("WEB3_PROVIDER_URL", default=None)

        if WEB3_PROVIDER_URL is not None:
            self.web3_service = Web3Service(
                provider_url=WEB3_PROVIDER_URL,
            )
        else:
            logging.warn(
                "⚠️ skipped Web3Service init, missing WEB3_PROVIDER_URL"
            )

        # -----------------------------------------------------------------

        EK_PLUGIN_ID = config("EK_PLUGIN_ID", default=None)

        if EK_PLUGIN_ID is not None:
            PORT = config("PORT", default=3001, cast=int)

            self.client_service = ClientService(
                port=PORT,
                plugin_id=EK_PLUGIN_ID
            )
        else:
            logging.warn("⚠️ skipped ClientService init, missing EK_PLUGIN_ID")

        # -----------------------------------------------------------------

        GOOGLE_SHEETS_CREDENTIALS_FILE = config("GOOGLE_SHEETS_CREDENTIALS_FILE", default=None)

        if GOOGLE_SHEETS_CREDENTIALS_FILE is not None:
            self.google_sheets_client = GoogleSheetsClient(
                credentials_file=GOOGLE_SHEETS_CREDENTIALS_FILE
            )
        else:
            logging.warn("⚠️ skipped GoogleSheetsClient init, missing GOOGLE_SHEETS_CREDENTIALS_FILE")
            
        # -----------------------------------------------------------------

        TWITTER_AUTH_TOKEN = config("TWITTER_AUTH_TOKEN", default=None)

        if TWITTER_AUTH_TOKEN is not None:
            self.twitter_client = TwitterClient(
                rest_client=self.rest_client,
                auth_token=TWITTER_AUTH_TOKEN
            )
        else:
            logging.warn("⚠️ skipped TwitterClient init, missing TWITTER_AUTH_TOKEN")
            
        # -----------------------------------------------------------------



from ekp_sdk.util.clean_null_terms import clean_null_terms


def Ul(items=None, style=None, class_name=None):
    return {
        "_type": "Ul",
        "props": clean_null_terms({
            "className": class_name,
            "style": style,
            "items": items or [],
        })
    }


def client_currency(event):
    if (event is None):
        return None

    if ("state" not in event.keys()):
        return None

    if ("client" not in event["state"].keys()):
        return None

    if ("selectedCurrency" not in event["state"]["client"].keys()):
        return None

    return event["state"]["client"]["selectedCurrency"]

def client_path(event):
    if (event is None):
        return None

    if ("state" not in event.keys()):
        return None

    if ("client" not in event["state"].keys()):
        return None

    if ("path" not in event["state"]["client"].keys()):
        return None

    return event["state"]["client"]["path"]

def form_value(event, form_name, property_name):
    if (event is None):
        return None

    if ("state" not in event.keys()):
        return None

    if ("forms" not in event["state"].keys()):
        return None

    if (form_name not in event["state"]["forms"].keys()):
        return None

    if (property_name not in event["state"]["forms"][form_name].keys()):
        return None

    return event["state"]["forms"][form_name][property_name]

def form_values(event, form_name):
    if (event is None):
        return None

    if ("state" not in event.keys()):
        return None

    if ("forms" not in event["state"].keys()):
        return None

    if (form_name not in event["state"]["forms"].keys()):
        return None

    return event["state"]["forms"][form_name]




from ekp_sdk.util.clean_null_terms import clean_null_terms


def Div(children=None, class_name=None, style=None, when=None, context=None):
    return {
        "_type": "Div",
        "props": clean_null_terms({
            "className": class_name,
            "children": children or [],
            "style": style,
            "when": when,
            "context": context
        })
    }


from ekp_sdk.util.clean_null_terms import clean_null_terms


def Badge(color, children, class_name=None):
    return {
        "_type": "Badge",
        "props": clean_null_terms({
            "children": children,
            "className": class_name,
            "color": color,
        })
    }



import json
from ekp_sdk.services.limiter import Limiter

from ekp_sdk.services.rest_client import RestClient


class CoingeckoService:
    def __init__(
        self,
        rest_client: RestClient
    ):
        self.base_url = "https://api.coingecko.com/api/v3"
        self.rest_client = rest_client
        self.limiter = Limiter(1000, 1)
    
    async def get_coin(self, id):
        url = f"{self.base_url}/coins/{id}"

        return await self.rest_client.get(url, limiter=self.limiter)
        
    async def get_coins(self):
        url = f"{self.base_url}/coins/list?include_platform=true"

        return await self.rest_client.get(url, limiter=self.limiter)
        
    async def get_coin_id_map(self, platform_id):

        url = f"{self.base_url}/coins/list?include_platform=true"

        response = await self.rest_client.get(url, limiter=self.limiter)

        map = {}

        for coin in response:
            if (platform_id not in coin["platforms"]):
                continue

            map[coin["platforms"][platform_id]] = coin["id"]

        return map
    
    async def get_coin_address_map(self, platform_id):

        url = f"{self.base_url}/coins/list?include_platform=true"

        response = await self.rest_client.get(url, limiter=self.limiter)

        map = {}

        for coin in response:
            if (platform_id not in coin["platforms"]):
                continue

            map[coin["id"]] = coin["platforms"][platform_id]

        return map
    
    async def get_historic_price(self, coin_id, date_str, fiat_id):

        url = f"{self.base_url}/coins/{coin_id}/history?date={date_str}"

        result = await self.rest_client.get(url, lambda data, text: data['market_data']['current_price'][fiat_id], limiter=self.limiter)

        return result

    async def get_latest_price(self, coin_id, fiat_id):

        url = f"{self.base_url}/simple/price?ids={coin_id}&vs_currencies={fiat_id}"

        result = await self.rest_client.get(url, lambda data, text: data[coin_id][fiat_id], limiter=self.limiter)

        return result

    async def get_coin_markets(self, page = 1, per_page = 50, vs_currency = "usd", category = None):

        url = f"{self.base_url}/coins/markets?vs_currency={vs_currency}&order=market_cap_desc&per_page={per_page}&page={page}&sparkline=false"
        
        if category:
            url += f"&category={category}"

        result = await self.rest_client.get(url, lambda data, text: data, limiter=self.limiter)

        return result



from ekp_sdk.util.clean_null_terms import clean_null_terms


def Image(src, style=None, class_name=None, when=None, tooltip=None):
    return {
        "_type": "Image",
        "props": clean_null_terms({
            "src": src,
            "style": style,
            "className": class_name,
            "when": when,
            "tooltip": tooltip
        })
    }


from ekp_sdk.util.clean_null_terms import clean_null_terms


def Avatar(icon, size=None, color=None):
    return {
        "_type": "Avatar",
        "props": clean_null_terms({
            "color": color,
            "icon": icon,
            "size": size
        })
    }


from ekp_sdk.services.limiter import Limiter
from ekp_sdk.services.rest_client import RestClient


class TwitterClient:
    def __init__(
        self,
        rest_client: RestClient,
        auth_token: str,
    ):
        self.auth_token = auth_token
        self.rest_client = rest_client
        self.limiter = Limiter(1000, 1)
        self.base_url = "https://api.twitter.com/1.1"

    async def get_user_info_by_screen_name(self, screen_name):
        url = f"{self.base_url}/users/show.json?screen_name={screen_name}"
        return await self.__get(url)

    async def __get(self, url, fn=lambda data, text: data, allowed_response_codes=[200]):
        headers = {"Authorization": f"Bearer {self.auth_token}"}

        result = await self.rest_client.get(
            url,
            fn,
            limiter=self.limiter,
            headers=headers,
            allowed_response_codes=allowed_response_codes
        )

        return result



from sqlalchemy import MetaData, create_engine
from sqlalchemy.orm import sessionmaker


class PgClient:
    def __init__(self, uri):
        self.meta_data = MetaData()
        self.engine = create_engine(uri)
        self.conn = self.engine.connect()
        Session = sessionmaker(bind=self.engine)
        self.session = Session()
        print("👍 Connected to Postgres")
